// Copyright 2017 Northern.tech AS
//
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.
package identity

import (
	"context"
)

type identityContextKeyType int

const (
	identityContextKey identityContextKeyType = 0
)

// FromContext extracts current identity from context.Context
func FromContext(ctx context.Context) *Identity {
	val := ctx.Value(identityContextKey)
	if v, ok := val.(*Identity); ok {
		return v
	}
	return nil
}

// WithContext adds identity to context `ctx` and returns the resulting context.
func WithContext(ctx context.Context, identity *Identity) context.Context {
	return context.WithValue(ctx, identityContextKey, identity)
}
