// Copyright 2017 Northern.tech AS
//
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.

package customheader

import "github.com/ant0ine/go-json-rest/rest"

// Add custom HTTP header to all server responses
type CustomHeaderMiddleware struct {
	HeaderName  string
	HeaderValue string
}

// MiddlewareFunc makes CustomHeaderMiddleware implement the Middleware interface.
func (c *CustomHeaderMiddleware) MiddlewareFunc(h rest.HandlerFunc) rest.HandlerFunc {
	return func(w rest.ResponseWriter, r *rest.Request) {

		if len(c.HeaderName) > 0 {
			w.Header().Add(c.HeaderName, c.HeaderValue)
		}

		// call the handler
		h(w, r)
	}
}
